#!/tools/gnu/bin/perl5 -w

#########################
# $Id: create_tb.pl,v 1.1 1999/10/29 20:41:16 spear Exp $
#########################
# $Log: create_tb.pl,v $
# Revision 1.1  1999/10/29 20:41:16  spear
# Initial revision
#
# Revision 1.1  1999/10/29 16:10:42  brianf
# Initial revision
#
#########################

# Created by: Brian Favela, ATI Research, Inc.

# Perl script to read in a top-level module and (eventually) create a
# bit-blasted testbench for applying vector stimulus through the PLI
# VCD reader for VCS.  The only hand-edits necessary are for clocks
# and reset.

# Create file pointer to read in requested top-level module
die "Please specify a top-level module name.\n" if !$ARGV[0];

die "Top-level entity '$ARGV[0].v' does not exist in this directory.\n" if
!-e "$ARGV[0].v";

$top_level_filename = "$ARGV[0].v";

open(TOPLEVEL, "$top_level_filename") || die "Error: could not open file
'$top_level_filename'\n";

$testbench_name = "tb_read_$ARGV[0].v";

die "File already exists: $testbench_name\n" if -e $testbench_name;

open(TBFILE, ">$testbench_name") || die "Error: could not create file
'$testbench_name'\n";

$input_count = 0;
$output_count = 0;
$multibit_count = 0;

@input_signals = "";
@output_signals = "";
@multibit_signals = "";
@multibit_sig_size = "";
@multibit_direction = "";

while(<TOPLEVEL>)
{
    if(/input\s*(.*)\s*;/)
    {
	$signal_name = $1;
	$_ = $1;
	if(/\[\s*(\d+\'d)?\s*(\d+)\s*:\s*(\d+\'d)?\s*(\d+)\s*\]\s*(\w+)\s*/)
	{
	    for($i = $2; $i >= $4; $i--)
	    {
		$input_signals[$input_count++] = "$5_$i";
	    }

	    $multibit_signals[$multibit_count] = "$5";
	    $multibit_direction[$multibit_count] = "i";
	    $multibit_sig_size[$multibit_count++] = "$2:$4";
	}
	else
	{
	    $input_signals[$input_count++] = $signal_name;
	}
    }
    if(/output\s*(.*);/)
    {
	$signal_name = $1;
	$_ = $1;
	if(/\[\s*(\d+\'d)?\s*(\d+)\s*:\s*(\d+\'d)?\s*(\d+)\s*\]\s*(.*)\s*/)
	{
	    for($i = $2; $i >= $4; $i--)
	    {
		$output_signals[$output_count++] = "$5_$i";
	    }

	    $multibit_signals[$multibit_count] = "$5";
	    $multibit_direction[$multibit_count] = "o";
	    $multibit_sig_size[$multibit_count++] = "$2:$4";
	}
	else
	{
	    $output_signals[$output_count++] = $signal_name;
	}
    }
}

print TBFILE "\n";
print TBFILE "/* Testbench for reading VCD file */\n\n";
print TBFILE " `timescale 1ns / 1ns\n\n";
print TBFILE "module tb_read_$ARGV[0];\n\n\n";
print TBFILE "reg  stop; // Signal to end VCD read\n";
print TBFILE "reg  compare; // Signal to pull reset\n";
print TBFILE "reg  mismatches; // Keep track of the number of failures\n";


for($i = 0; $i < $input_count; $i++)
{
    print TBFILE "reg  $input_signals[$i];\n";
}

for($i = 0; $i < $output_count - 1; $i++)
{
    print TBFILE "wire $output_signals[$i];\n";
    print TBFILE "reg $output_signals[$i]_VCD;\n";
}

print TBFILE "wire $output_signals[$output_count - 1];\n\n";
print TBFILE "reg $output_signals[$output_count - 1]_VCD;\n\n";

for($i = 0; $i < $multibit_count; $i++)
{
    print TBFILE "wire \[$multibit_sig_size[$i]\] $multibit_signals[$i]";
    if($multibit_direction[$i] eq "i")
    {
	print TBFILE ";\n";
    }
    else
    {
	print TBFILE "_VCD;\n";
	print TBFILE "wire \[$multibit_sig_size[$i]\]
$multibit_signals[$i];\n";
    }
}

for($i = 0; $i < $multibit_count; $i++)
{
    if($multibit_direction[$i] eq "o")
    {
	print TBFILE "assign $multibit_signals[$i]_VCD = {";
	($left, $right) = split /:/, $multibit_sig_size[$i];
	for($j = $left; $j >= $right; $j--)
	{
	    print TBFILE "$multibit_signals[$i]" . "_${j}_VCD";
	    if($j != $right)
	    {
		print TBFILE ", ";
	    }
	}
	print TBFILE "};\n";
	
	for($j = $left; $j >= $right; $j--)
	{
	    print TBFILE "assign $multibit_signals[$i]" . "_$j =
$multibit_signals[$i]\[$j\];\n";
	}
    }
    else
    {
	print TBFILE "assign $multibit_signals[$i] = {";
	($left, $right) = split /:/, $multibit_sig_size[$i];
	for($j = $left; $j >= $right; $j--)
	{
	    print TBFILE "$multibit_signals[$i]" . "_${j}";
	    if($j != $right)
	    {
		print TBFILE ", ";
	    }
	}
	print TBFILE "};\n";
    }
}


print TBFILE "$ARGV[0] u$ARGV[0] (\n";

for($i = 0; $i < $multibit_count; $i++)
{
    print TBFILE ".$multibit_signals[$i]($multibit_signals[$i]),\n";
}

for($i = 0; $i < $input_count; $i++)
{
    $_ = $input_signals[$i];
    if(/([\w+_]+)_(\d+)$/)
    {
# don't do anything for multibit signals
    }
    else
    {
	print TBFILE ".$input_signals[$i]($input_signals[$i]),\n";
    }
}

for($i = 0; $i < $output_count; $i++)
{
    $_ = $output_signals[$i];
    if(/([\w+_]+)_(\d+)$/)
    {
# don't do anything for multibit signals
    }
    else
    {
	print TBFILE ".$output_signals[$i]($output_signals[$i])";
	if($i != $output_count - 1)
	{
	    print TBFILE ",\n";
	}
    }
}



print TBFILE "\n);\n";

print TBFILE "	initial\n";
print TBFILE "	  begin\n";
print TBFILE "	    mismatches = 0;\n";
print TBFILE "	    stop = 0;\n";
print TBFILE "	    \$timeformat(-9,0,\"ns\",5);\n";

if(!defined $ARGV[1] || $ARGV[1] eq "")
{
    print TBFILE "	    \$vcdread(\"VCD_FILE\");\n";
}
else
{
    print "Setting VCD file to $ARGV[1]\n";
    print TBFILE "          \$vcdread(\"$ARGV[1]\");\n";
}
#print TBFILE "	    \$vcdread_options(\"verbose\");\n";
print TBFILE "	    // Compare            \"VCD name\", simulation name\n";
print TBFILE "	    \$vcdread_signal_pairs(\n";


#print NEWTOP "$ARGV[0] u$ARGV[0] (\n";


if(!defined $ARGV[2] || $ARGV[2] eq "")
{
    $SCOPE_DECLARATION = "SCOPE_DECLARATION";
}
else
{
    print "Setting scope to $ARGV[2].\n";
    $SCOPE_DECLARATION = $ARGV[2];
}

for($i = 0; $i < $input_count; $i++)
{
    $_ = $input_signals[$i];
    tr/A-Z/a-z/;
    print TBFILE "            \"$SCOPE_DECLARATION.$_\",
$input_signals[$i],\n";
}

for($i = 0; $i < $output_count; $i++)
{
    $_ = $output_signals[$i];
    tr/A-Z/a-z/;
    print TBFILE "            \"$SCOPE_DECLARATION.$_\",
$output_signals[$i]_VCD";
    if($i + 1 != $output_count)
    {
	print TBFILE ",\n";
    }
}

print TBFILE "      );\n";
print TBFILE "	    \$display(\"Entering loop\");\n";
print TBFILE "	    wait (stop == 1)\n";
print TBFILE "	    \$finish;\n";
print TBFILE "	end\n";

print TBFILE "// Set a flag after reset has been deasserted\n";

print TBFILE "	 initial\n";
print TBFILE "	  begin\n";
print TBFILE "	      compare = 0;\n";
print TBFILE "	      wait (iRESET == 1'b1)\n";
print TBFILE "	      wait (iRESET == 1'b0)\n";
print TBFILE "	      wait (iRESET == 1'b1)\n";
print TBFILE "	      compare = 1;\n";
print TBFILE "	      \$display(\"Out of reset\");\n";
print TBFILE "	  end\n";

print TBFILE "\n\n";
print TBFILE "    always @(posedge iSCLK)\n";
print TBFILE "    begin\n";
print TBFILE "\n\n\n";

for($i = 0; $i < $multibit_count; $i++)
{
    if($multibit_direction[$i] eq "o")
    {
	print TBFILE "       if($multibit_signals[$i] !=
$multibit_signals[$i]_VCD)\n";
	print TBFILE "       begin\n";
	print TBFILE "           \$display(\"Mismatch at
$multibit_signals[$i] at time %t. Sim: %h, VCD: %h\", \$realtime,
$multibit_signals[$i], $multibit_signals[$i]_VCD);\n";
	print TBFILE "           mismatches = mismatches + 1;\n";
	print TBFILE "       end\n\n";
    }

}

for($i = 0; $i < $output_count; $i++)
{
    $_ = $output_signals[$i];
    if(/([\w+_]+)_(\d+)$/)
    {
# don't do anything for multibit signals
    }
    else
    {
	print TBFILE "       if($output_signals[$i] !=
$output_signals[$i]_VCD)\n";
	print TBFILE "       begin\n";
	print TBFILE "           \$display(\"Mismatch at $output_signals[$i]
at time %t.\ Sim: %h, VCD: %h\", \$realtime, $output_signals[$i],
$output_signals[$i]_VCD);\n";
	print TBFILE "           mismatches = mismatches + 1;\n";
	print TBFILE "       end\n\n";
    }
}

print TBFILE "    end\n";
print TBFILE "\n\n\nendmodule\n";

print "Done!\n";



