/* Counter is bidirectional, with the load signal controling the direction */

`timescale 1ns/1ns
module dff(q, qb, clock, d, reset);
	input clock, d, reset;
	output q, qb;

	nand #2 n1 (cf, dl, cbf),
		n2 (cbf, clock, cf, reset),
		n3 (q, cbf, qb),
		n4 (qb, dbl, q, reset);
	nand #1 n5 (dl, d, dbl, reset),
		n6 (dbl, dl, clock, cbf);
endmodule /* dff */
   
// o = sel ? a : b
module mux(o, sel, a, b);
   output o;
   input a, b, sel;
   
   or #1 (o, as, bs);
   not (sb, sel);
   and (as, a, sel);
   and (bs, b, sb);
   
endmodule /* mux */
   
   

module counter (d, clock, load, reset);
   inout [3:0] d;
   input clock, load, reset;
   wire [3:0] m, q, qb;
   
   mux m0 (m[0], load, d[0], qb[0]);
   dff d0 (q[0], qb[0], clock, m[0], reset);
   xor (q01, q[0], q[1]);
   mux m1 (m[1], load, d[1], q01);
   dff d1 (q[1], qb[1], clock, m[1], reset);
   and (q2a, q[2], qb[0]);
   and (q2b, q[2], qb[1]);
   and (q2c, qb[2], q[1], q[0]);
   or (q2_next, q2a, q2b, q2c);
   mux m2 (m[2], load, d[2], q2_next);
   dff d2 (q[2], qb[2], clock, m[2], reset);
   and (q3a, q[3], qb[0]);
   and (q3b, q[3], qb[1]);
   and (q3c, q[3], qb[2]);
   and (q3d, qb[3], q[2], q[1], q[0]);
   or  (q3_next, q3a, q3b, q3c, q3d);
   mux m3 (m[3], load, d[3], q3_next);
   dff d3 (q[3], qb[3], clock, m[3], reset);

   bufif0 b0 (d[0], q[0], load);
   bufif0 b1 (d[1], q[1], load);
   bufif0 b2 (d[2], q[2], load);
   bufif0 b3 (d[3], q[3], load);
   
endmodule /* counter */
