/* Testbench for reading VCD file */
 `timescale 100ps/10ps
module test;
   reg [3:0] d_reg;
   reg clock, reset, compare, load, stop, strobe;
   wire [3:0] d;
   integer mismatches;
   real real_big, real_small;

   counter i1 (d, clock, load, reset);

   assign d = load ? d_reg : 4'bz;
   initial
    begin
       mismatches = 0;
       $timeformat(-9,0,"ns",5);
       // $vcdread_options("start_after_0");  // Must come before $vcdread call
       $vcdread("verilog.dump");
       // $vcdread_options("verbose");
       // Compare            "VCD name", simulation name
       $vcdread_signal_pairs("test.clock", clock,
			     "test.d", d_reg,
			     "test.load", load,
			     "test.reset", reset,
			     "test.stop", stop,
			     "test.strobe", strobe,
			     "test.r_big", real_big,
			     "test.r_small", real_small);
     
       wait (stop == 1) 
	$display("\nSummary: There were %0d mismatches.\n", mismatches);
       $finish;
    end
   
   // Set a flag after reset has been deasserted
   initial
    begin
       compare = 0;
       wait (reset == 1'b0)
       wait (reset == 1'b1)
       compare = 1;
    end
   
   // Print the signals just before the clock rises
   // The d and d_reg signals should match value
   always @ (posedge strobe)
    begin
       $display("@%t  d=%b, d_reg=%b, clock=%b, load=%b, reset=%b",
	         $realtime, d, d_reg, clock, load, reset);
       if (compare && (d !== d_reg))
	begin
	   $display("**** d (%b) !== d_reg (%b) expected value", d, d_reg);
	   mismatches = mismatches + 1;
	end
    end
   
endmodule /* test */

